import os
import time
import allure
import uiautomator2 as u2
from qrunner.utils.config import conf
from qrunner.utils.log import logger
from baseImage import Image, Rect


class AndroidDriver(object):
    # _instance = {}
    #
    # def __new__(cls, serial_no=None):
    #     if not serial_no:
    #         serial_no = conf.get_name('app', 'serial_no')
    #     if serial_no not in cls._instance:
    #         cls._instance[serial_no] = super().__new__(cls)
    #     return cls._instance[serial_no]

    def __init__(self, serial_no=None, pkg_name=None):
        if serial_no is None:
            serial_no = conf.get_item('app', 'serial_no')
        self.serial_no = serial_no
        if pkg_name is None:
            pkg_name = conf.get_item('app', 'pkg_name')
        self.pkg_name = pkg_name

        logger.info(f'启动 android driver for {self.serial_no}')
        self.d = u2.connect(self.serial_no)
        self.session = None

    # @classmethod
    # def get_instance(cls, serial_no=None):
    #     """Create singleton"""
    #     if serial_no not in cls._instance:
    #         logger.info(f'[{serial_no}] Create android driver singleton')
    #         return AndroidDriver(serial_no)
    #     return AndroidDriver._instance[serial_no]

    # @classmethod
    # def get_remote_instance(cls, server_url, token):
    #     device = Device(server_url, token)
    #     d = u2.connect(device.get_device())
    #     return d, device

    def uninstall_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'卸载应用: {pkg_name}')
        self.d.app_uninstall(pkg_name)

    def install_app(self, apk_path):
        logger.info(f'覆盖安装应用: {apk_path}')
        self.d.app_install(apk_path)

    def new_install_app(self, apk_path):
        logger.info(f'先卸载再安装应用: {apk_path}')
        self.uninstall_app(self.pkg_name)
        self.d.app_install(apk_path)

    def start_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'启动应用: {pkg_name}')
        self.d.app_start(pkg_name)

    def force_start_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'强制启动应用: {pkg_name}')
        self.d.app_start(pkg_name, stop=True)

    def stop_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'退出应用: {pkg_name}')
        self.d.app_stop(pkg_name)

    def stop_all_app(self):
        logger.info(f'退出所有应用')
        self.d.app_stop_all()

    def stop_app_list(self, app_list: list):
        logger.info(f'退出多个应用: {app_list}')
        self.d.app_stop_all(excludes=app_list)

    def clear_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'清除应用缓存: {pkg_name}')
        self.d.app_clear(pkg_name)

    def get_driver_info(self):
        logger.info(f'获取连接信息')
        return self.d.info

    def get_app_info(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'获取指定应用信息: {pkg_name}')
        info = self.d.app_info(pkg_name)
        logger.info(info)
        return info

    def get_current_app(self):
        logger.info(f'获取当前应用信息')
        return self.d.app_current()

    def save_app_icon(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'获取应用icon并保存到当前目录')
        img = self.d.app_icon(pkg_name)
        img.save(f'{pkg_name}-icon.png')

    def get_running_apps(self):
        logger.info(f'获取所有正在运行的应用')
        app_list = self.d.app_list_running()
        logger.info(app_list)
        return app_list

    def get_app_list(self):
        logger.info(f'获取所有已安装的应用')
        app_list = self.d.app_list()
        logger.info(app_list)
        return app_list

    def wait_app_running(self, pkg_name=None, front=True, timeout=20):
        """
        等待应用运行
        @param pkg_name: 应用包名
        @param front: 是否前台运行
        @param timeout: 等待时间
        @return: 应用pid
        """
        if not pkg_name:
            pkg_name = self.pkg_name
        pid = self.d.app_wait(pkg_name, front=front, timeout=timeout)
        if not pid:
            logger.info(f'{pkg_name} is not running')
        else:
            logger.info(f'{pkg_name} pid is {pid}')
        return pid

    def wait_activity(self, activity_name, timeout=10):
        """
        等待activity运行
        @param activity_name: activity名称，.ApiDemos
        @param timeout: 超时时间
        @return: True or False
        """
        logger.info(f'等待activity {activity_name}')
        flag = self.d.wait_activity(activity_name, timeout=timeout)
        logger.info(flag)
        return flag

    def push(self, src_path, target_path, mode=None):
        """
        把电脑本地文件上传到手机上
        @param src_path: 电脑本地文件，foo.txt
        @param target_path: 手机目录，/sdcard/
        @param mode: 需要修改的权限，0o755
        @return:
        """
        logger.info(f'把{src_path} push到手机 {target_path}目录')
        if mode is not None:
            self.d.push(src_path, target_path)
        else:
            self.d.push(src_path, target_path, mode=mode)

    def pull(self, src_path, target_path):
        """
        把手机上的文件下载到电脑
        @param src_path: 手机文件，/sdcard/tmp.txt
        @param target_path: 电脑目录，tmp.txt
        @return:
        """
        logger.info(f'{src_path} pull到电脑 {target_path}目录')
        self.d.pull(src_path, target_path)

    def check(self):
        logger.info('检查并维持设备端守护进程处于运行状态')
        self.d.healthcheck()

    def open_url(self, url):
        """
        通过url打开web页面或者app schema
        @param url: 页面url，https://www.baidu.com，taobao://taobao.com
        @return:
        """
        logger.info(f'打开链接: {url}')
        self.d.open_url(url)

    def shell(self, cmd, timeout=60):
        """
        执行短周期shell脚本
        @param cmd: shell字符串或list，pwd，["ls", "-l"]
        @param timeout: 超时时间
        @return:
        """
        logger.info(f'执行shell命令: {cmd}')
        output, exit_code = self.d.shell(cmd, timeout=timeout)
        return output, exit_code

    def start_session(self, pkg_name=None):
        """
        启动应用并生成session
        @param pkg_name: 应用包名
        @return:
        """
        logger.info(f'启动{pkg_name}session')
        if not pkg_name:
            pkg_name = self.pkg_name
        self.session = self.d.session(pkg_name)

    def stop_session(self):
        logger.info(f'关闭session并停止应用')
        self.session.close()

    def check_session(self):
        logger.info(f'检查session是否可用')
        is_running = self.session.running()
        logger.info(is_running)
        return is_running

    def screenshot(self, file_name, rect=None):
        """
        截图并保存到预定路径并上传到allure报告
        @param file_name: foo.png or fool
        @param rect: 需要标记控件的左上角坐标和宽高
        @return:
        """
        # 把文件名处理成test.png的样式
        if '.' in file_name:
            file_name = file_name.split(r'.')[0]
        # 截图并保存到当前目录的images文件夹中
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
        file_path = os.path.join(img_dir,
                                 f'{time_str}-{file_name}.png')
        logger.info(f'截图保存至: {file_path}')
        self.d.screenshot(file_path)

        # 给图片打上标记
        if rect is not None:
            img = Image(file_path)
            img.rectangle(Rect(*rect), color=(0, 0, 255), thickness=3)
            img.imwrite(file_path)

        # 截图并上传到allure报告
        logger.info(f'截图上传至allure报告')
        allure.attach.file(file_path,
                           attachment_type=allure.attachment_type.PNG,
                           name=f'{time_str}-{file_name}.png')

    # def upload_pic(self, filename):
    #     self.screenshot('tmp.png')
    #     allure.attach.file(
    #         'tmp.png',
    #         attachment_type=allure.attachment_type.PNG,
    #         name=f'{filename}-{get_time()}'
    #     )
    #     os.remove('tmp.png')

    # def input_password(self, text):
    #     self.d(focused=True).set_text(text)

    def get_page_content(self):
        return self.d.dump_hierarchy()

    def get_window_size(self):
        return self.d.window_size()

    def get_serial(self):
        logger.info('获取设备id')
        serial = self.d.serial
        logger.info(serial)
        return serial

    def get_device_info(self):
        logger.info(f'获取设备信息')
        info = self.d.device_info
        logger.info(info)
        return info

    def screen_on(self):
        logger.info('点亮屏幕')
        self.d.screen_on()

    def screen_off(self):
        logger.info('关闭屏幕')
        self.d.screen_off()

    def get_screen_status(self):
        logger.info('获取屏幕点亮状态')
        status = self.d.info.get('screenOn')
        logger.info(status)
        return status

    def unlock(self):
        logger.info('解锁手机')
        self.d.unlock()

    def press(self, key):
        """
        点击原生自带按键
        @param key: 按键名，支持：home、back、left、right、up、down、center、menu、search、enter、delete、recent、volume_up、
                    volume_down、volume_mute、camera、power
        @return:
        """
        logger.info(f'点击key: {key}')
        self.press(key)

    def click(self, x, y):
        logger.info(f'单击坐标: {x},{y}')
        self.d.click(x, y)

    def click_alert(self, alert_list: list):
        logger.info(f'批量点击弹窗: {alert_list}')
        with self.d.watch_context() as ctx:
            for alert in alert_list:
                ctx.when(alert).click()
            ctx.wait_stable()

    def double_click(self, x, y):
        logger.info(f'双击坐标: {x},{y}')
        self.d.double_click(x, y)

    def long_click(self, x, y):
        logger.info(f'长按坐标: {x},{y}')
        self.d.long_click(x, y)

    def swipe(self, sx, sy, ex, ey):
        logger.info(f'从坐标{sx},{sy} 滑到 {ex},{ey}')
        self.d.swipe(sx, sy, ex, ey)

    def swipe_left(self, scale=0.9):
        logger.info('往左滑动')
        self.d.swipe_ext('left', scale=scale)

    def swipe_right(self, scale=0.9):
        logger.info('往右滑动')
        self.d.swipe_ext('right', scale=scale)

    def swipe_up(self, scale=0.8):
        logger.info('往上滑动')
        self.d.swipe_ext('up', scale=scale)

    def swipe_down(self, scale=0.8):
        logger.info('往下滑动')
        self.d.swipe_ext('down', scale=scale)

    def scroll_down_fast(self):
        logger.info('快速往下滑动')
        self.d(scrollable=True).fling()

    def scroll_down_slow(self, step=50):
        """
        通过step控制滑动速度
        @param step:
        @return:
        """
        logger.info('慢速往下滑动')
        self.d(scrollable=True).scroll(steps=step)

    def scroll_up_fast(self):
        logger.info('快速往上滑动')
        self.d(scrollable=True).fling.vert.backward()

    def scroll_up_down(self, step):
        """
        通过step控制滑动速度
        @param step:
        @return:
        """
        logger.info('慢速往下滑动')
        self.d(scrollable=True).scroll.vert.backward(steps=step)

    def scroll_bottom_fast(self):
        logger.info('快速滑到底部')
        self.d(scrollable=True).fling.toEnd()

    def scroll_bottom_slow(self, step=50):
        """
        通过step控制滑动速度
        @param step:
        @return:
        """
        logger.info('慢速滑到底部')
        self.d(scrollable=True).scroll.toEnd(steps=step)

    def scroll_top_fast(self):
        logger.info('快速滑到顶部')
        self.d(scrollable=True).fling.toBeginning()

    def scroll_top_slow(self, step=50):
        """
        通过step控制滑动速度
        @param step:
        @return:
        """
        logger.info('慢速滑到顶部')
        self.d(scrollable=True).scroll.toBeginning(steps=step)

    def scroll_to(self, *args, **kwargs):
        logger.info(f'滑动到元素: {kwargs}')
        self.d(scrollable=True).scroll.to(*args, **kwargs)

    def drag(self, sx, sy, ex, ey):
        logger.info(f'从坐标{sx},{sy} 拖动到坐标{ex},{ey}')
        self.d.drag(sx, sy, ex, ey)

    def set_password(self, text, clear=True):
        logger.info(f'输入: {text}')
        self.d.set_fastinput_ime(True)
        if clear:
            self.d.clear_text()
        self.d(focused=True).set_text(text)
        self.d.set_fastinput_ime(False)

    def set_ori_left(self):
        logger.info('屏幕向右边转动')
        self.d.set_orientation("l")

    def set_ori_right(self):
        logger.info('屏幕向左边转动')
        self.d.set_orientation('r')

    def set_ori_natural(self):
        logger.info('屏幕恢复原始转向')
        self.d.set_orientation('n')

    def start_record(self, file_name='output'):
        logger.info('开始录制')
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%m%d%H%M%S')
        file_path = os.path.join(img_dir,
                                 f'{file_name}-{time_str}.mp4')
        self.d.screenrecord(file_path)

    def stop_record(self):
        logger.info('停止录屏')
        self.d.screenrecord.stop()









import os.path
import sys

api_run_content = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='api',
        base_url='https://www.qizhidao.com'
    )
"""

android_run_content = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='android',
        serial_no='UJK0220521066836',
        pkg_name='com.qizhidao.clientapp'
    )
"""

ios_run_content = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='ios',
        serial_no='00008101-000E646A3C29003A',
        pkg_name='com.qizhidao.company'
    )
"""

browser_run_content = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='web',
        base_url='https://patents-pre.qizhidao.com'
    )
"""

case_content_android = """import qrunner


@qrunner.story('首页')
class TestAdrSample(qrunner.TestCase):
    
    @qrunner.title('从首页进入我的页')
    def test_go_my(self):
        self.click(resourceId='id/bottom_btn', desc='广告关闭按钮')
        self.click(resourceId='id/bottom_view', index=3, desc='底部导航-我的')
        self.assertText('我的订单', desc='我的订单标题')
"""

case_content_ios = """import qrunner


@qrunner.story('首页')
class TestIosSample(qrunner.TestCase):

    @qrunner.title('从首页进入我的页')
    def test_go_my(self):
        self.click(label='我的', desc='底部导航-我的')
        self.assertText('我的订单', desc='我的订单标题')
"""

case_content_web = """import qrunner


@qrunner.story('专利检索')
class TestWebSample(qrunner.TestCase):
    
    @qrunner.title('专利简单检索')
    def test_patent_search(self):
        self.open_url('https://patents-pre.qizhidao.com/', desc='专利首页')
        self.set_text('无人机', xpath='//*[@id="driver-home-step1"]', desc='专利搜索框')
        self.click(id_='driver-home-step2', desc='搜索按钮')
        self.assertTitle('无人机专利检索-企知道', desc='专利搜索页')
"""

case_content_api = """import qrunner


@qrunner.story('PC站首页')
class TestApiSample(qrunner.TestCase):

    @qrunner.title('查询PC站首页banner列表')
    @qrunner.file_data('card_type', 'data.json')
    def test_getToolCardListForPc(self, card_type):
        path = '/api/qzd-bff-app/qzd/v1/home/getToolCardListForPc'
        pay_load = {"type": card_type}
        self.post(path, json=pay_load)
        self.assertEq('code', 0)
"""

require_content = """qrunner
"""

ignore_content = "\n".join(
    ["__pycache__/*", "*.pyc", ".idea/*", ".DS_Store", "allure-results", "report"]
)

data_content = """{
  "card_type": [0, 1, 2]
}
"""


def init_scaffold_project(subparsers):
    parser = subparsers.add_parser("start", help="Create a new project with template structure.")
    parser.add_argument('-t', '--platform', dest='platform', type=str, default='api', help='测试平台: api、android、ios、web')
    return parser


def create_scaffold(platform):
    """ create scaffold with specified project name.
    """

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    project_name = f'{platform}_demo'
    create_folder(project_name)
    create_file(
        os.path.join(project_name, ".gitignore"),
        ignore_content,
    )
    create_file(
        os.path.join(project_name, "requirements.txt"),
        require_content,
    )
    create_folder(os.path.join(project_name, 'test_dir'))
    create_file(
        os.path.join(project_name, 'test_dir', "__init__.py"),
        '',
    )
    if platform == 'api':
        create_file(
            os.path.join(project_name, "run.py"),
            api_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_api_sample.py"),
            case_content_api,
        )
    elif platform == 'android':
        create_file(
            os.path.join(project_name, "run.py"),
            android_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_adr_sample.py"),
            case_content_android,
        )
    elif platform == 'ios':
        create_file(
            os.path.join(project_name, "run.py"),
            ios_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_ios_sample.py"),
            case_content_ios,
        )
    elif platform == 'web':
        create_file(
            os.path.join(project_name, "run.py"),
            browser_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_web_sample.py"),
            case_content_web,
        )
    else:
        print(f'不支持的平台: {platform}')
        sys.exit()
    create_folder(os.path.join(project_name, 'test_data'))
    create_file(
        os.path.join(project_name, 'test_data', "data.json"),
        data_content,
    )
    create_folder(os.path.join(project_name, 'report'))
    return 0


def main_scaffold_project(args):
    sys.exit(create_scaffold(args.platform))




import inspect
import json
import os
import time
from multiprocessing.dummy import Pool as ThreadPool
from typing import Union
from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector
from qrunner.utils.log import logger
from qrunner.utils.config import conf
from qrunner.core.android.driver import AndroidDriver
from qrunner.utils.exceptions import NoSuchElementException
from qrunner.utils.decorate import step


class AndroidElement(object):
    """
    安卓元素定义
    """
    def __init__(self, driver=None, **kwargs):
        # 从kwargs中删掉并返回index
        self._index = kwargs.pop('index', 0)
        self.desc = kwargs.pop('desc', '')
        # 参数初始化
        self._xpath = kwargs.get('xpath', '')
        self._kwargs = kwargs
        self._element: Union[UiObject, XPathSelector]
        self._driver: AndroidDriver = driver
        self._serial_no = self._driver.serial_no
        self._pkg_name = self._driver.pkg_name
        self._resourceId = kwargs.pop('resourceId', '')
        if self._resourceId:
            kwargs['resourceId'] = f'{self._pkg_name}:{self._resourceId}'
        self._d = self._driver.d

    def handle_alert(self):
        """
        根据不同定位方式进行点击
        @return:
        """
        def click_alert(loc):
            if 'id/' in loc:
                element = self._d(resourceId=f'{self._pkg_name}:{loc}')
            elif '//' in loc:
                element = self._d.xpath(loc)
            else:
                element = self._d(text=loc)
            element.click_exists(timeout=1)

        # 多线程执行点击过程
        alert_list = conf.get_item('app', 'android_alert')
        print(f'异常弹窗列表: {alert_list}')
        pool = ThreadPool(len(alert_list))
        if alert_list:
            pool.map(click_alert, alert_list)
        else:
            logger.debug("弹窗设置列表为空")
        logger.debug('处理异常弹窗完成')

    def _find_element(self, retry=3, timeout=3):
        """
        循环查找元素，查找失败先处理弹窗后重试，后面再考虑xpath要不要用.all()改造一下
        @param retry: 重试次数
        @param timeout: 每次查找超时时间
        @return: 找到的元素列表
        """
        logger.info(f'查找元素: {self._kwargs},{self._index}')
        self._element = self._d.xpath(self._xpath) if \
            self._xpath else self._d(**self._kwargs)[self._index]
        self.handle_alert()  # 处理异常弹窗
        while not self._element.wait(timeout=timeout):
            if retry > 0:
                retry -= 1
                logger.warning(f'重试 查找元素： {self._kwargs},{self._index}')
                self.handle_alert()  # 处理异常弹窗
            else:
                frame = inspect.currentframe().f_back
                caller = inspect.getframeinfo(frame)
                logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
                return None
        return self._element

    def get_element(self, retry=3, timeout=3):
        """
        针对元素定位失败的情况，抛出NoSuchElementException异常
        @param retry:
        @param timeout:
        @return:
        """
        element = self._find_element(retry=retry, timeout=timeout)
        if element is None:
            self._driver.screenshot(f'[控件 {self.desc} 定位失败]')
            raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
        return element

    @property
    def info(self):
        logger.info(f'获取元素: {self._kwargs} 的所有信息')
        return self.get_element().info

    @property
    def text(self):
        logger.info(f'获取元素: {self._kwargs} 的文本')
        return self.get_element().info.get('text')

    @property
    def bounds(self):
        logger.info(f'获取元素: {self._kwargs} 的坐标')
        return self.get_element().info.get('bounds')

    @property
    def rect(self):
        logger.info(f'获取元素: {self._kwargs} 左上角的坐标以及宽高')
        bounds = self.get_element().info.get('bounds')
        x = bounds['left']
        y = bounds['top']
        width = bounds['right'] - x
        height = bounds['bottom'] - y
        return [x, y, width, height]

    @property
    def visibleBounds(self):
        logger.info(f'获取元素: {self._kwargs} 的可见坐标')
        return self.get_element().info.get('visibleBounds')

    @property
    def focusable(self):
        logger.info(f'获取元素: {self._kwargs} 是否聚焦')
        return self.get_element().info.get('focusable')

    @property
    def selected(self):
        logger.info(f'获取元素: {self._kwargs} 是否选中')
        return self.get_element().info.get('selected')

    def child(self, *args, **kwargs):
        logger.info(f'获取元素 {self._kwargs},{self._index} 的子元素{kwargs}')
        return self.get_element().child(*args, **kwargs)

    def brother(self, *args, **kwargs):
        logger.info(f'获取元素 {self._kwargs},{self._index} 的兄弟元素{kwargs}')
        return self.get_element().sibling(*args, **kwargs)

    def left(self, *args, **kwargs):
        logger.info(f'获取元素 {self._kwargs} 左边的元素 {kwargs}')
        return self.get_element().left(*args, **kwargs)

    def right(self, *args, **kwargs):
        logger.info(f'获取元素 {self._kwargs} 右边的元素 {kwargs}')
        return self.get_element().right(*args, **kwargs)

    def up(self, *args, **kwargs):
        logger.info(f'获取元素 {self._kwargs} 上边的元素 {kwargs}')
        return self.get_element().up(*args, **kwargs)

    def down(self, *args, **kwargs):
        logger.info(f'获取元素 {self._kwargs} 下边的元素 {kwargs}')
        return self.get_element().down(*args, **kwargs)

    def exists(self, timeout=1):
        logger.info(f'判断元素是否存在: {self._kwargs},{self._index}')
        element = self._find_element(retry=0, timeout=timeout)
        if element is None:
            # self._driver.screenshot(f'元素定位失败')
            return False
        return True

    def click(self):
        logger.info(f'点击元素: {self._kwargs},{self._index}')
        element = self.get_element()
        print('点击元素')
        element.click()
        print('点击成功')

    def click_exists(self):
        logger.info(f'存在才点击元素: {self._kwargs},{self._index}')
        if self.exists():
            self.click()

    def click_gone(self):
        logger.info(f'等元素 {self._kwargs} 消失后再点击')
        flag = self.get_element().click_gone()
        logger.info(flag)
        return flag

    def wait_gone(self, timeout=3):
        logger.info(f'等元素 {self._kwargs} 消失')
        flag = self.get_element().wait_gone(timeout=timeout)
        logger.info(flag)
        return flag

    def long_click(self):
        logger.info(f'长按元素 {self._kwargs}')
        self.get_element().long_click()

    def set_text(self, text):
        logger.info(f'输入文本: {text}')
        self.get_element().set_text(text)

    def clear_text(self):
        logger.info('清除文本')
        self.get_element().clear_text()

    def drag_to(self, *args, **kwargs):
        logger.info(f'从当前元素{self._kwargs},{self._index}, 拖动到元素: {kwargs}')
        self.get_element().drag_to(*args, **kwargs)

    def swipe_left(self):
        logger.info(f'往左滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("left")

    def swipe_right(self):
        logger.info(f'往右滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("right")

    def swipe_up(self):
        logger.info(f'往上滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("up")

    def swipe_down(self):
        logger.info(f'往下滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("down")






from .. import Provider as AddressProvider


class Provider(AddressProvider):

    city_formats = ("{{city_name}}",)

    street_name_formats = ("{{street_name}}",)
    street_address_formats = ("{{street_name}} {{building_number}}",)
    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    building_number_formats = ("###", "##", "#", "#a", "#b", "#c")

    postcode_formats = ("####",)

    cities = (
        "Ajdovščina",
        "Bled",
        "Bovec",
        "Brežice",
        "Celje",
        "Cerknica",
        "Črnomelj",
        "Domžale",
        "Dravograd",
        "Gornja Radgona",
        "Gornji Grad",
        "Grosuplje",
        "Hrastnik",
        "Idrija",
        "Ilirska Bistrica",
        "Izola",
        "Jesenice",
        "Kamnik",
        "Kobarid",
        "Kočevje",
        "Koper",
        "Kostanjevica na Krki",
        "Kranj",
        "Krško",
        "Laško",
        "Lenart v Slovenskih goricah",
        "Lendava",
        "Litija",
        "Ljubljana",
        "Ljutomer",
        "Logatec",
        "Maribor",
        "Medvode",
        "Mengeš",
        "Metlika",
        "Mežica",
        "Murska Sobota",
        "Nova Gorica",
        "Novo mesto",
        "Ormož",
        "Piran",
        "Postojna",
        "Prevalje",
        "Ptuj",
        "Radeče",
        "Radovljica",
        "Ravne na Koroškem",
        "Ribnica",
        "Rogaška Slatina",
        "Ruše",
        "Sevnica",
        "Sežana",
        "Slovenj Gradec",
        "Slovenska Bistrica",
        "Slovenske Konjice",
        "Šempeter pri Gorici",
        "Šentjur",
        "Škofja Loka",
        "Šoštanj",
        "Tolmin",
        "Trbovlje",
        "Trebnje",
        "Tržič",
        "Turnišče",
        "Velenje",
        "Vipava",
        "Vipavski Križ",
        "Višnja Gora",
        "Vrhnika",
        "Zagorje ob Savi",
        "Žalec",
        "Železniki",
        "Žiri",
    )

    streets = (
        "Abramova ulica",
        "Adamičeva ulica",
        "Adamič-Lundrovo nabrežje",
        "Ajdovščina",
        "Aleševa ulica",
        "Alešovčeva ulica",
        "Aljaževa ulica",
        "Ambrožev trg",
        "Ameriška ulica",
        "Andrićeva ulica",
        "Anžurjeva ulica",
        "Apihova ulica",
        "Argentinska ulica",
        "Arharjeva cesta",
        "Arkova ulica",
        "Artačeva ulica",
        "Aškerčeva cesta",
        "Avčinova ulica",
        "Avsečeva ulica",
        "Avstrijska ulica",
        "Avšičeva cesta",
        "Ažmanova ulica",
        "Babičeva ulica",
        "Badjurova ulica",
        "Balinarska pot",
        "Baragova ulica",
        "Barjanska cesta",
        "Bavdkova ulica",
        "Baznikova ulica",
        "Bazoviška ulica",
        "Beethovnova ulica",
        "Belačeva ulica",
        "Beljaška ulica",
        "Berčičeva ulica",
        "Berčonova pot",
        "Berdajsova ulica",
        "Bernekerjeva ulica",
        "Bernikova ulica",
        "Betettova cesta",
        "Bezenškova ulica",
        "Bežigrad",
        "Bičevje",
        "Bilečanska ulica",
        "Bitenčeva ulica",
        "Bizjakova ulica",
        "Bizjanova ulica",
        "Bizovški štradon",
        "Blasnikova ulica",
        "Blasov breg",
        "Bleiweisova cesta",
        "Bobenčkova ulica",
        "Bobrova ulica",
        "Bognarjeva pot",
        "Bohinjčeva ulica",
        "Bohoričeva ulica",
        "Boletova ulica",
        "Bolgarska ulica",
        "Borovniška ulica",
        "Borštnikov trg",
        "Borutova ulica",
        "Božičeva ulica",
        "Brankova ulica",
        "Bratinova ulica",
        "Bratislavska cesta",
        "Bratov Jakopičev ulica",
        "Bratov Kunovarjev ulica",
        "Bravničarjeva ulica",
        "Brdnikova ulica",
        "Breg",
        "Bregarjeva ulica",
        "Breznikova ulica",
        "Brglezov štradon",
        "Brilejeva ulica",
        "Brodarjev trg",
        "Brodska cesta",
        "Burnikova ulica",
        "Cankarjev vrh",
        "Cankarjevo nabrežje",
        "Carja Dušana ulica",
        "Celarčeva ulica",
        "Celjska ulica",
        "Celovška cesta",
        "Cerkniška ulica",
        "Cerutova ulica",
        "Cesta Andreja Bitenca",
        "Cesta Ceneta Štuparja",
        "Cesta Dolomitskega odreda",
        "Cesta II. grupe odredov",
        "Cesta Ljubljanske brigade",
        "Cesta na Bellevue",
        "Cesta na Bokalce",
        "Cesta na Brinovec",
        "Cesta na Brod",
        "Cesta na Ježah",
        "Cesta na Kope",
        "Cesta na Laze",
        "Cesta na Loko",
        "Cesta na Mesarico",
        "Cesta na Ozare",
        "Cesta na Poljane",
        "Cesta na Prevoje",
        "Cesta na Urh",
        "Cesta na Vrhovce",
        "Cesta slov. kmečkih uporov",
        "Cesta Urške Zatlerjeve",
        "Cesta v Dvor",
        "Cesta v Gameljne",
        "Cesta v Hrastje",
        "Cesta v hrib",
        "Cesta v Kleče",
        "Cesta v Kostanj",
        "Cesta v Legarico",
        "Cesta v Mestni log",
        "Cesta v Pečale",
        "Cesta v Prod",
        "Cesta v Rožno dolino",
        "Cesta v Šmartno",
        "Cesta v Zeleni log",
        "Cesta v Zgornji log",
        "Cesta vstaje",
        "Cesta 24. junija",
        "Cesta 25 talcev",
        "Cesta 27. aprila",
        "Chengdujska cesta",
        "Chopinov prehod",
        "Cigaletova ulica",
        "Cilenškova ulica",
        "Cimermanova ulica",
        "Cimpermanova ulica",
        "Cizejeva ulica",
        "Clevelandska ulica",
        "Colnarjeva ulica",
        "Cvetlična pot",
        "Čampova ulica",
        "Čanžekova ulica",
        "Čargova ulica",
        "Čebelarska ulica",
        "Čehova ulica",
        "Čepelnikova ulica",
        "Čepovanska ulica",
        "Čerinova ulica",
        "Černigojeva ulica",
        "Černivčeva ulica",
        "Červanova ulica",
        "Čevljarska ulica",
        "Čižmanova ulica",
        "Čopova ulica",
        "Črna pot",
        "Črnuška cesta",
        "Črtomirova ulica",
        "Čučkova ulica",
        "Dajnkova ulica",
        "Dalmatinova ulica",
        "Danile Kumarjeve ulica",
        "Dečkova ulica",
        "Dečmanova ulica",
        "Delakova ulica",
        "Demšarjeva cesta",
        "Derčeva ulica",
        "Dergančeva ulica",
        "Dermotova ulica",
        "Detelova ulica",
        "Devinska ulica",
        "Devova ulica",
        "Divjakova ulica",
        "Do proge",
        "Dobrajčeva ulica",
        "Dobrdobska ulica",
        "Dolenjska cesta",
        "Dolgi breg",
        "Dolgi most",
        "Dolharjeva ulica",
        "Dolinarjeva ulica",
        "Dolinškova ulica",
        "Dolničarjeva ulica",
        "Dolomitska ulica",
        "Drabosnjakova ulica",
        "Draga",
        "Draveljska ulica",
        "Dražgoška ulica",
        "Drenikov vrh",
        "Drenikova ulica",
        "Dunajska cesta",
        "Dvojna ulica",
        "Dvorakova ulica",
        "Dvorni trg",
        "Eipprova ulica",
        "Ellerjeva ulica",
        "Emonska cesta",
        "Erbežnikova ulica",
        "Erjavčeva cesta",
        "Fabianijeva ulica",
        "Fani Grumove ulica",
        "Ferberjeva ulica",
        "Filipičeva ulica",
        "Flajšmanova ulica",
        "Flandrova ulica",
        "Forsterjeva ulica",
        "Franketova ulica",
        "Frankopanska ulica",
        "Frenkova pot",
        "Friškovec",
        "Funtkova ulica",
        "Fužinska cesta",
        "Gabrov trg",
        "Gača",
        "Galičeva ulica",
        "Galjevica",
        "Gallusovo nabrežje",
        "Gasilska cesta",
        "Gasparijeva ulica",
        "Gašperšičeva ulica",
        "Gerbičeva ulica",
        "Gestrinova ulica",
        "Glavarjeva ulica",
        "Gledališka stolba",
        "Glinška ulica",
        "Glinškova ploščad",
        "Glonarjeva ulica",
        "Gmajnice",
        "Gobarska pot",
        "Godeževa ulica",
        "Gola Loka",
        "Golarjeva ulica",
        "Goljarjeva pot",
        "Golouhova ulica",
        "Goriška ulica",
        "Gorjančeva ulica",
        "Gorjupova ulica",
        "Gornji Rudnik I",
        "Gornji Rudnik II",
        "Gornji Rudnik III",
        "Gornji trg",
        "Goropečnikova ulica",
        "Gortanova ulica",
        "Gospodinjska ulica",
        "Gosposka ulica",
        "Gosposvetska cesta",
        "Govekarjeva ulica",
        "Gozdna pot",
        "Grablovičeva ulica",
        "Gradišče",
        "Gradnikova ulica",
        "Grafenauerjeva ulica",
        "Grajski drevored",
        "Grajzerjeva ulica",
        "Gramozna pot",
        "Grassellijeva ulica",
        "Gregorčičeva ulica",
        "Gregorinova ulica",
        "Grintovška ulica",
        "Grobeljca",
        "Grobeljska pot",
        "Groharjeva cesta",
        "Groznikova ulica",
        "Grška ulica",
        "Grško",
        "Gruberjevo nabrežje",
        "Grudnovo nabrežje",
        "Gubčeva ulica",
        "Gunceljska cesta",
        "Gustinčarjeva ulica",
        "Gustinčičeva ulica",
        "Hacetova ulica",
        "Hafnerjeva ulica",
        "Hajdrihova ulica",
        "Hauptmanca",
        "Hladilniška pot",
        "Hladnikova cesta",
        "Hlebčeva ulica",
        "Hotimirova ulica",
        "Hradeckega cesta",
        "Hranilniška ulica",
        "Hribarjevo nabrežje",
        "Hribernikova ulica",
        "Hribovska pot",
        "Hrvaška ulica",
        "Hrvatski trg",
        "Hubadova ulica",
        "Hudourniška pot",
        "Idrijska ulica",
        "Igriška ulica",
        "Ilešičeva ulica",
        "Ilovški štradon",
        "Industrijska cesta",
        "Ingličeva ulica",
        "Italijanska ulica",
        "Izletniška ulica",
        "Ižanska cesta",
        "Jakčeva ulica",
        "Jakhljeva ulica",
        "Jakopičev drevored",
        "Jakopičevo sprehajališče",
        "Jakšičeva ulica",
        "Jalnova ulica",
        "Jamova cesta",
        "Janežičeva cesta",
        "Janova ulica",
        "Janševa ulica",
        "Jarčeva ulica",
        "Jarnikova ulica",
        "Jarše",
        "Jarška cesta",
        "Javorškova ulica",
        "Jazbečeva pot",
        "Jelinčičeva ulica",
        "Jenkova ulica",
        "Jensenova ulica",
        "Jerajeva ulica",
        "Jeranova ulica",
        "Jesenkova ulica",
        "Jesihov štradon",
        "Jezerska ulica",
        "Ježa",
        "Ježica",
        "Joškov štradon",
        "Jurčičev trg",
        "Jurčkova cesta",
        "Juričeva ulica",
        "Juvanova ulica",
        "K reaktorju",
        "Kadilnikova ulica",
        "Kajuhova ulica",
        "Kalingerjeva ulica",
        "Kalinova ulica",
        "Kaminova ulica",
        "Kamniška ulica",
        "Kamnogoriška cesta",
        "Kančeva ulica",
        "Kanonijeva cesta",
        "Kantetova ulica",
        "Kapusova ulica",
        "Kardeljeva ploščad",
        "Karingerjeva ulica",
        "Karunova ulica",
        "Kastelčeva ulica",
        "Kašeljska cesta",
        "Kavadarska cesta",
        "Kavčičeva ulica",
        "Kavškova ulica",
        "Kekčeva ulica",
        "Kermaunerjeva ulica",
        "Kernova cesta",
        "Kerševanova ulica",
        "Keržičeva ulica",
        "Kettejeva ulica",
        "Kladezna ulica",
        "Klančarjeva ulica",
        "Kleče",
        "Klemenova ulica",
        "Kleparska steza",
        "Ključavničarska ulica",
        "Klunova ulica",
        "Kmečka pot",
        "Knafljev prehod",
        "Knezov štradon",
        "Knezova ulica",
        "Knobleharjeva ulica",
        "Koblarjeva ulica",
        "Kocbekova ulica",
        "Kocenova ulica",
        "Kocjanova ulica",
        "Kočenska ulica",
        "Kodrova ulica",
        "Kogojeva ulica",
        "Kogovškova ulica",
        "Kokaljeva ulica",
        "Kolarjeva ulica",
        "Kolesarska pot",
        "Koleševa ulica",
        "Kolinska ulica",
        "Kolmanova ulica",
        "Kolodvorska ulica",
        "Komanova ulica",
        "Komenskega ulica",
        "Kongresni trg",
        "Kopališka ulica",
        "Kopitarjeva ulica",
        "Kopna pot",
        "Koprska ulica",
        "Koreninova ulica",
        "Koroška ulica",
        "Korotanska ulica",
        "Kosančeva ulica",
        "Koseskega ulica",
        "Koseška cesta",
        "Kosmačeva ulica",
        "Kosova ulica",
        "Kosovelova ulica",
        "Koširjeva ulica",
        "Kotnikova ulica",
        "Kovačeva ulica",
        "Kovaška ulica",
        "Kovinarska ulica",
        "Kozakova ulica",
        "Kozinova ulica",
        "Kozlarjeva pot",
        "Koželjeva ulica",
        "Krakovski nasip",
        "Kraljeva ulica",
        "Kranerjeva ulica",
        "Kraška ulica",
        "Kratka pot",
        "Kratka steza",
        "Kregarjeva ulica",
        "Kreljeva ulica",
        "Kremžarjeva ulica",
        "Krimska ulica",
        "Krištofova ulica",
        "Kriva pot",
        "Krivec",
        "Križevniška soteska",
        "Križna ulica",
        "Krmčeva ulica",
        "Krmeljeva ulica",
        "Kropova ulica",
        "Krošljeva ulica",
        "Krovska ulica",
        "Krožna pot",
        "Kržičeva ulica",
        "Kudrova ulica",
        "Kuhljeva cesta",
        "Kumerdejeva ulica",
        "Kumerjeve ulica",
        "Kumrovška ulica",
        "Kurilniška ulica",
        "Kurirska ulica",
        "Kusoldova ulica",
        "Kuštrinova ulica",
        "Kuzeletova ulica",
        "Kuzmičeva ulica",
        "Lahova pot",
        "Lajovčeva ulica",
        "Laknerjeva ulica",
        "Lakotence",
        "Lampetova ulica",
        "Lamutova ulica",
        "Langusova ulica",
        "Latinski trg",
        "Lavrinova ulica",
        "Layerjeva ulica",
        "Lazarjeva ulica",
        "Legatova ulica",
        "Lemeževa ulica",
        "Lepi pot",
        "Lepodvorska ulica",
        "Leskovičeva ulica",
        "Letališka cesta",
        "Levarjeva ulica",
        "Levičnikova ulica",
        "Levstikov trg",
        "Levstikova ulica",
        "Linhartov podhod",
        "Linhartova cesta",
        "Lipahova ulica",
        "Litijska cesta",
        "Litostrojska cesta",
        "Livada",
        "Livarska ulica",
        "Ločnikarjeva ulica",
        "Lončarska steza",
        "Lorenzova cesta",
        "Lovrenčičeva ulica",
        "Lovska ulica",
        "Lovšetova ulica",
        "Lubejeva ulica",
        "Luize Pesjakove ulica",
        "Lunačkova ulica",
        "Mačja steza",
        "Mačkov kot",
        "Mačkova ulica",
        "Madžarska ulica",
        "Magistrova ulica",
        "Maistrova ulica",
        "Majaronova ulica",
        "Majde Vrhovnikove ulica",
        "Majorja Lavriča ulica",
        "Makucova ulica",
        "Mala ulica",
        "Mala vas",
        "Malejeva ulica",
        "Malenškova ulica",
        "Malgajeva ulica",
        "Mali štradon",
        "Mali trg",
        "Malnarjeva ulica",
        "Marčenkova ulica",
        "Marentičeva ulica",
        "Mareška pot",
        "Marice Kovačeve ulica",
        "Marincljeva ulica",
        "Marinovševa cesta",
        "Maroltova ulica",
        "Martina Krpana ulica",
        "Martinčeva ulica",
        "Martinova ulica",
        "Marušičeva ulica",
        "Masarykova cesta",
        "Matjanova pot",
        "Matjaževa ulica",
        "Maurerjeva ulica",
        "Mazovčeva pot",
        "Med hmeljniki",
        "Medarska ulica",
        "Medenska cesta",
        "Medveščkova ulica",
        "Mekinčeva ulica",
        "Melikova ulica",
        "Mencingerjeva ulica",
        "Merčnikova ulica",
        "Merosodna ulica",
        "Mesesnelova ulica",
        "Mestni trg",
        "Meškova ulica",
        "Metelkova ulica",
        "Miheličeva cesta",
        "Mihov štradon",
        "Miklavčeva ulica",
        "Miklošičeva cesta",
        "Mikuževa ulica",
        "Milčetova pot",
        "Mire Lenardičeve ulica",
        "Mirje",
        "Mirna pot",
        "Mislejeva ulica",
        "Mizarska pot",
        "Mladinska ulica",
        "Mlake",
        "Mlinska pot",
        "Močnikova ulica",
        "Mokrška ulica",
        "Molekova ulica",
        "Moškričeva ulica",
        "Mrharjeva ulica",
        "Mrzelova ulica",
        "Murkova ulica",
        "Murnikova ulica",
        "Murnova ulica",
        "Muzejska ulica",
        "Na cvetači",
        "Na delih",
        "Na dolih",
        "Na gaju",
        "Na gmajni",
        "Na Herši",
        "Na jami",
        "Na klančku",
        "Na Korošci",
        "Na Palcah",
        "Na požaru",
        "Na produ",
        "Na Rojah",
        "Na Stolbi",
        "Na Straški vrh",
        "Na Trati",
        "Na Žalah",
        "Nade Ovčakove ulica",
        "Nadgoriška cesta",
        "Nahlikova ulica",
        "Nahtigalova ulica",
        "Nanoška ulica",
        "Nazorjeva ulica",
        "Nebotičnikov prehod",
        "Nedohova ulica",
        "Njegoševa cesta",
        "Nova ulica",
        "Novakova pot",
        "Novakova ulica",
        "Novi trg",
        "Novinarska ulica",
        "Novo naselje",
        "Novo Polje, cesta I",
        "Novo Polje, cesta III",
        "Novo Polje, cesta IV",
        "Novo Polje, cesta V",
        "Novo Polje, cesta VI",
        "Novo Polje, cesta VII",
        "Novo Polje, cesta X",
        "Novo Polje, cesta XI",
        "Novo Polje, cesta XII",
        "Novo Polje, cesta XIV",
        "Novo Polje, cesta XIX",
        "Novo Polje, cesta XVI",
        "Novo Polje, cesta XVII",
        "Novo Polje, cesta XXI",
        "Novo Polje, cesta XXIII",
        "Novosadska ulica",
        "Ob daljnovodu",
        "Ob dolenjski železnici",
        "Ob Farjevcu",
        "Ob Ljubljanici",
        "Ob Mejašu",
        "Ob potoku",
        "Ob pristanu",
        "Ob Savi",
        "Ob studencu",
        "Ob zdravstvenem domu",
        "Ob zeleni jami",
        "Ob zelenici",
        "Ob žici",
        "Obirska ulica",
        "Obrežna steza",
        "Obrije",
        "Ocvirkova ulica",
        "Ogrinčeva ulica",
        "Okiškega ulica",
        "Omahnova ulica",
        "Omejčeva ulica",
        "Omersova ulica",
        "Oražnova ulica",
        "Orlova ulica",
        "Osenjakova ulica",
        "Osojna pot",
        "Osojna steza",
        "Osterčeva ulica",
        "Ovčakova ulica",
        "Pahorjeva ulica",
        "Palmejeva ulica",
        "Papirniška pot",
        "Park Ajdovščina",
        "Park Arturo Toscanini",
        "Parmova ulica",
        "Parmska cesta",
        "Partizanska ulica",
        "Pavlovčeva ulica",
        "Pavšičeva ulica",
        "Pečarjeva ulica",
        "Pečnik",
        "Pečnikova ulica",
        "Pegamova ulica",
        "Perčeva ulica",
        "Periška cesta",
        "Perkova ulica",
        "Peršinova cesta",
        "Pesarska cesta",
        "Pestotnikova ulica",
        "Peščena pot",
        "Petkova ulica",
        "Petkovškovo nabrežje",
        "Petrčeva ulica",
        "Pilonova ulica",
        "Pionirska pot",
        "Pipanova pot",
        "Pirnatova ulica",
        "Planinska cesta",
        "Planinškova ulica",
        "Plečnikov podhod",
        "Plemljeva ulica",
        "Plešičeva ulica",
        "Pleteršnikova ulica",
        "Pločanska ulica",
        "Pod akacijami",
        "Pod bregom",
        "Pod bresti",
        "Pod bukvami",
        "Pod Debnim vrhom",
        "Pod gabri",
        "Pod gozdom",
        "Pod hrasti",
        "Pod hribom",
        "Pod hruško",
        "Pod jelšami",
        "Pod jezom",
        "Pod ježami",
        "Pod Kamno gorico",
        "Pod klancem",
        "Pod lipami",
        "Pod topoli",
        "Pod Trančo",
        "Pod turnom",
        "Pod vrbami",
        "Podgornikova ulica",
        "Podgorska cesta",
        "Podgrajska cesta",
        "Podjunska ulica",
        "Podlimbarskega ulica",
        "Podmilščakova ulica",
        "Podrožniška pot",
        "Podsmreška cesta",
        "Podutiška cesta",
        "Pogačarjev trg",
        "Pohlinova ulica",
        "Poklukarjeva ulica",
        "Polakova ulica",
        "Polanškova ulica",
        "Poljanska cesta",
        "Polje",
        "Polje, cesta I",
        "Polje, cesta II",
        "Polje, cesta III",
        "Polje, cesta VI",
        "Polje, cesta VIII",
        "Polje, cesta X",
        "Polje, cesta XIV",
        "Polje, cesta XL",
        "Polje, cesta XLII",
        "Polje, cesta XLVI",
        "Polje, cesta XVI",
        "Polje, cesta XVIII",
        "Polje, cesta XXII",
        "Polje, cesta XXIV",
        "Polje, cesta XXVI",
        "Polje, cesta XXX",
        "Polje, cesta XXXII",
        "Polje, cesta XXXIV",
        "Polje, cesta XXXVIII",
        "Poljedelska ulica",
        "Poljska pot",
        "Porentova ulica",
        "Posavskega ulica",
        "Postojnska ulica",
        "Pot do šole",
        "Pot Draga Jakopiča",
        "Pot heroja Trtnika",
        "Pot k igrišču",
        "Pot k ribniku",
        "Pot k Savi",
        "Pot k sejmišču",
        "Pot k studencu",
        "Pot na Breje",
        "Pot na Drenikov vrh",
        "Pot na Golovec",
        "Pot na goro",
        "Pot na Gradišče",
        "Pot na Grič",
        "Pot na Labar",
        "Pot na mah",
        "Pot na most",
        "Pot na Orle",
        "Pot na Visoko",
        "Pot na Zduše",
        "Pot Rdečega križa",
        "Pot v boršt",
        "Pot v Čeželj",
        "Pot v dolino",
        "Pot v Goričico",
        "Pot v hribec",
        "Pot v mejah",
        "Pot v Mlake",
        "Pot v Podgorje",
        "Pot v Zeleni gaj",
        "Pot za Brdom",
        "Pot za razori",
        "Potokarjeva ulica",
        "Potrčeva ulica",
        "Povšetova ulica",
        "Prašnikarjeva ulica",
        "Praznikova ulica",
        "Pražakova ulica",
        "Pred Savljami",
        "Predjamska cesta",
        "Predor pod Gradom",
        "Preglov trg",
        "Prekmurska ulica",
        "Prelčeva ulica",
        "Preloge",
        "Premrlova ulica",
        "Preradovićeva ulica",
        "Preserska ulica",
        "Prešernov trg",
        "Prešernova cesta",
        "Pretnarjeva ulica",
        "Pri borštu",
        "Pri brvi",
        "Pri malem kamnu",
        "Pri mostiščarjih",
        "Pribinova ulica",
        "Prijateljeva ulica",
        "Primorska ulica",
        "Prinčičeva ulica",
        "Prisojna ulica",
        "Prištinska ulica",
        "Privoz",
        "Proletarska cesta",
        "Prule",
        "Prušnikova ulica",
        "Prvomajska ulica",
        "Pšatnik",
        "Pšatska pot",
        "Ptujska ulica",
        "Pučnikova ulica",
        "Puharjeva ulica",
        "Puhova ulica",
        "Puhtejeva ulica",
        "Puterlejeva ulica",
        "Putrihova ulica",
        "Raičeva ulica",
        "Rakovniška ulica",
        "Rakuševa ulica",
        "Ramovševa ulica",
        "Ravbarjeva ulica",
        "Ravna pot",
        "Ravnikova ulica",
        "Razgledna steza",
        "Reber",
        "Reboljeva ulica",
        "Rečna ulica",
        "Regentova cesta",
        "Resljeva cesta",
        "Reška ulica",
        "Ribičičeva ulica",
        "Ribji trg",
        "Ribniška ulica",
        "Rimska cesta",
        "Rjava cesta",
        "Robbova ulica",
        "Robičeva ulica",
        "Rodičeva ulica",
        "Rojčeva ulica",
        "Romavhova ulica",
        "Rosna pot",
        "Rotarjeva ulica",
        "Rovšnikova ulica",
        "Rozmanova ulica",
        "Rožanska ulica",
        "Rožičeva ulica",
        "Rožna dolina, cesta I",
        "Rožna dolina, cesta III",
        "Rožna dolina, cesta IV",
        "Rožna dolina, cesta V",
        "Rožna dolina, cesta VI",
        "Rožna dolina, cesta VIII",
        "Rožna dolina, cesta X",
        "Rožna dolina, cesta XII",
        "Rožna dolina, cesta XIII",
        "Rožna dolina, cesta XV",
        "Rožna dolina, cesta XVII",
        "Rožna ulica",
        "Rudnik I",
        "Rudnik II",
        "Rudnik III",
        "Runkova ulica",
        "Ruska ulica",
        "Rutarjeva ulica",
        "Sadinja vas",
        "Sajovčeva ulica",
        "Samova ulica",
        "Saškova ulica",
        "Sattnerjeva ulica",
        "Savinova ulica",
        "Savinškova ulica",
        "Savlje",
        "Savska cesta",
        "Sedejeva ulica",
        "Selanov trg",
        "Selanova ulica",
        "Setnikarjeva ulica",
        "Seunigova ulica",
        "Simončičeva ulica",
        "Siva pot",
        "Skapinova ulica",
        "Sketova ulica",
        "Skopčeva ulica",
        "Skrbinškova ulica",
        "Slape",
        "Slapnikova ulica",
        "Slavčja ulica",
        "Slomškova ulica",
        "Slovenčeva ulica",
        "Slovenska cesta",
        "Smoletova ulica",
        "Smrekarjeva ulica",
        "Smrtnikova ulica",
        "Snebersko nabrežje",
        "Snežniška ulica",
        "Snojeva ulica",
        "Sojerjeva ulica",
        "Sončna pot",
        "Sostrska cesta",
        "Soška ulica",
        "Soteška pot",
        "Soussenska ulica",
        "Sovretova ulica",
        "Spodnji Rudnik I",
        "Spodnji Rudnik II",
        "Spodnji Rudnik III",
        "Spodnji Rudnik V",
        "Spomeniška pot",
        "Srebrničeva ulica",
        "Srednja pot",
        "Stadionska ulica",
        "Staničeva ulica",
        "Stara Ježica",
        "Stara slovenska ulica",
        "Stare Črnuče",
        "Stari trg",
        "Stegne",
        "Steletova ulica",
        "Sternadova ulica",
        "Stiška ulica",
        "Stolpniška ulica",
        "Stoženska ulica",
        "Stožice",
        "Stražarjeva ulica",
        "Streliška ulica",
        "Stritarjeva ulica",
        "Strmeckijeva ulica",
        "Strmi pot",
        "Strniševa cesta",
        "Strossmayerjeva ulica",
        "Strugarska ulica",
        "Strupijevo nabrežje",
        "Suhadolčanova ulica",
        "Sulčja ulica",
        "Svetčeva ulica",
        "Šarhova ulica",
        "Šentjakob",
        "Šentviška ulica",
        "Šerkova ulica",
        "Šestova ulica",
        "Šibeniška ulica",
        "Šinkov štradon",
        "Šišenska cesta",
        "Šivičeva ulica",
        "Škerljeva ulica",
        "Škofova ulica",
        "Škrabčeva ulica",
        "Šlandrova ulica",
        "Šlosarjeva ulica",
        "Šmarna gora",
        "Šmartinska cesta",
        "Šmartno",
        "Španova pot",
        "Španska ulica",
        "Štajerska cesta",
        "Štebijeva cesta",
        "Štefančeva ulica",
        "Štembalova ulica",
        "Štepanjska cesta",
        "Štepanjsko nabrežje",
        "Štirnova ulica",
        "Štradon čez Prošco",
        "Štrekljeva ulica",
        "Študentovska ulica",
        "Štukljeva cesta",
        "Štula",
        "Šturmova ulica",
        "Šubičeva ulica",
        "Šumarjeva ulica",
        "Švabićeva ulica",
        "Švarova ulica",
        "Švegljeva cesta",
        "Tabor",
        "Tacenska cesta",
        "Tavčarjeva ulica",
        "Tbilisijska ulica",
        "Tesarska ulica",
        "Teslova ulica",
        "Tesna ulica",
        "Tesovnikova ulica",
        "Tiha ulica",
        "Tiranova ulica",
        "Tischlerjeva ulica",
        "Tivolska cesta",
        "Tkalska ulica",
        "Tobačna ulica",
        "Tolminska ulica",
        "Tomačevo",
        "Tomačevska cesta",
        "Tomažičeva ulica",
        "Tometova ulica",
        "Tominškova ulica",
        "Tomišeljska ulica",
        "Toplarniška ulica",
        "Topniška ulica",
        "Torkarjeva ulica",
        "Tratnikova ulica",
        "Travniška ulica",
        "Trbeže",
        "Trdinova ulica",
        "Trebušakova ulica",
        "Trg francoske revolucije",
        "Trg mladih",
        "Trg mladinskih delov. brigad",
        "Trg narodnih herojev",
        "Trg prekomorskih brigad",
        "Trg republike",
        "Trg 9. maja",
        "Trinkova ulica",
        "Trnovčeva ulica",
        "Trnovska ulica",
        "Trpinčeva ulica",
        "Trstenjakova ulica",
        "Trtnikova ulica",
        "Tržaška cesta",
        "Tržna ulica",
        "Tugomerjeva ulica",
        "Turnerjeva ulica",
        "Turnsko nabrežje",
        "Udvančeva ulica",
        "Ulica aktivistov",
        "Ulica Alme Sodnik",
        "Ulica Andreja Kumarja",
        "Ulica Angelce Ocepkove",
        "Ulica Angele Ljubičeve",
        "Ulica borca Petra",
        "Ulica borcev za severno mejo",
        "Ulica bratov Bezlajev",
        "Ulica bratov Blanč",
        "Ulica bratov Jančar",
        "Ulica bratov Komel",
        "Ulica bratov Kraljič",
        "Ulica bratov Martinec",
        "Ulica bratov Novak",
        "Ulica bratov Rozmanov",
        "Ulica bratov Škofov",
        "Ulica bratov Učakar",
        "Ulica bratov Židan",
        "Ulica Dušana Kraigherja",
        "Ulica Ernesta Kramerja",
        "Ulica Franca Nebca",
        "Ulica Francke Jerasove",
        "Ulica Franja Novaka",
        "Ulica gledališča BTC",
        "Ulica Goce Delčeva",
        "Ulica Gubčeve brigade",
        "Ulica Hermana Potočnika",
        "Ulica Ivana Roba",
        "Ulica Ivanke Kožuh",
        "Ulica Ivice Pirjevčeve",
        "Ulica Janeza Pavla II.",
        "Ulica Janeza Rožiča",
        "Ulica Jožeta Jame",
        "Ulica Jožeta Japlja",
        "Ulica Jožeta Mirtiča",
        "Ulica Konrada Babnika",
        "Ulica Koroškega bataljona",
        "Ulica Lizike Jančarjeve",
        "Ulica Lojzeta Spacala",
        "Ulica Lovre Klemenčiča",
        "Ulica Malči Beličeve",
        "Ulica Marije Drakslerjeve",
        "Ulica Marije Hvaličeve",
        "Ulica Marje Boršnikove",
        "Ulica Marka Šlajmerja",
        "Ulica Milana Majcna",
        "Ulica Milke Kerinove",
        "Ulica Minke Bobnar",
        "Ulica Mirka Jurce",
        "Ulica Mirka Tomšiča",
        "Ulica Miroslava Turka",
        "Ulica Molniške čete",
        "Ulica na Grad",
        "Ulica Nade Čamernikove",
        "Ulica Olge Mohorjeve",
        "Ulica padlih borcev",
        "Ulica Pariške komune",
        "Ulica Pohorskega bataljona",
        "Ulica Polonce Čude",
        "Ulica prvoborcev",
        "Ulica Rezke Dragarjeve",
        "Ulica Rezke Klopčič",
        "Ulica Rudolfa Janežiča",
        "Ulica Staneta Severja",
        "Ulica Štefke Zbašnikove",
        "Ulica talcev",
        "Ulica Tončke Čečeve",
        "Ulica v Kokovšek",
        "Ulica Vide Pregarčeve",
        "Ulica Vladimirja Trampuža",
        "Ulica Zore Ragancinove",
        "Ulica Žanke Erjavec",
        "Ulica 15. aprila",
        "Ulica 15. maja",
        "Ulica 24. avgusta",
        "Ulica 4. julija",
        "Ulica 7. septembra",
        "Ulica 9. junija",
        "Uršičev štradon",
        "Usnjarska ulica",
        "V Češnjico",
        "V dolini",
        "V Karlovce",
        "V Karlovce",
        "V Kladeh",
        "V Murglah",
        "V Sige",
        "V Varde",
        "V Zalar",
        "Vagajeva ulica",
        "Valjavčeva ulica",
        "Valvasorjeva ulica",
        "Vandotova ulica",
        "Vaška pot",
        "Večna pot",
        "Vegova ulica",
        "Velebitska ulica",
        "Veliki štradon",
        "Velikovška ulica",
        "Velnarjeva ulica",
        "Verovškova ulica",
        "Veršičeva ulica",
        "Veselova ulica",
        "Videmska ulica",
        "Vidergarjeva ulica",
        "Vidičeva ulica",
        "Vidovdanska cesta",
        "Vilharjev podhod",
        "Vilharjeva cesta",
        "Vinterca",
        "Vipavska ulica",
        "Vipotnikova ulica",
        "Viška cesta",
        "Vižmarska pot",
        "Vodmatska ulica",
        "Vodmatski trg",
        "Vodna steza",
        "Vodnikova cesta",
        "Vodnikovo naselje",
        "Vodovodna cesta",
        "Vogelna ulica",
        "Vojkova cesta",
        "Volaričeva ulica",
        "Vošnjakova ulica",
        "Vozna pot na Grad",
        "Vožarski pot",
        "Vrazov trg",
        "Vrbovec",
        "Vrbska ulica",
        "Vregova ulica",
        "Vrhovci, cesta I",
        "Vrhovci, cesta II",
        "Vrhovci, cesta III",
        "Vrhovci, cesta IX",
        "Vrhovci, cesta V",
        "Vrhovci, cesta VI",
        "Vrhovci, cesta X",
        "Vrhovci, cesta XI",
        "Vrhovci, cesta XII",
        "Vrhovci, cesta XIV",
        "Vrhovci, cesta XIX",
        "Vrhovci, cesta XV",
        "Vrhovci, cesta XVII",
        "Vrhovci, cesta XVIII",
        "Vrhovci, cesta XX",
        "Vrhovci, cesta XXII",
        "Vrhovci, cesta XXVI",
        "Vrhovci, cesta XXVIII",
        "Vrhovci, cesta XXXII",
        "Vrhovčeva ulica",
        "Vrhovnikova ulica",
        "Vrtača",
        "Vrtna ulica",
        "Vrtnarska cesta",
        "Vulčeva ulica",
        "Vzajemna ulica",
        "Windischerjeva ulica",
        "Wolfova ulica",
        "Za Garažami",
        "Za gasilskim domom",
        "Za Gradom",
        "Za krajem",
        "Za opekarno",
        "Za partizanskim domom",
        "Za progo",
        "Za vasjo",
        "Zadnikarjeva ulica",
        "Zadobrovška cesta",
        "Zadružna ulica",
        "Zajčeva pot",
        "Zajčevi dvori",
        "Zakotnikova ulica",
        "Zalaznikova ulica",
        "Zaletelova ulica",
        "Zaloška cesta",
        "Zarnikova ulica",
        "Zasavska cesta",
        "Zatišje",
        "Zavetiška ulica",
        "Završje",
        "Zbašnikova ulica",
        "Zdešarjeva cesta",
        "Zelena pot",
        "Zelenova ulica",
        "Zeljarska ulica",
        "Zevnikova ulica",
        "Zidarjev štradon",
        "Ziherlova ulica",
        "Zlatek",
        "Znamenjska ulica",
        "Zofke Kvedrove ulica",
        "Zoisova cesta",
        "Zupanova ulica",
        "Zvezda",
        "Zvezdarska ulica",
        "Zvezna ulica",
        "Žabarjeva ulica",
        "Žabjak",
        "Žalska ulica",
        "Žaucerjeva ulica",
        "Žeje",
        "Železna cesta",
        "Železnikarjeva ulica",
        "Žerjalova ulica",
        "Židankova ulica",
        "Židovska steza",
        "Židovska ulica",
        "Živaličeva ulica",
        "Živinozdravska ulica",
        "Žolgerjeva ulica",
    )

    states = (
        "Pomurksa",
        "Podravska",
        "Koroška",
        "Savinjska",
        "Zasavska",
        "Spodnjeposavska",
        "Jugovzhodna Slovenija",
        "Osrednjeslovenska",
        "Gorenjska",
        "Notranjsko - kraška",
        "Goriška",
        "Obalno - kraška",
    )

    countries = (
        "Afganistan",
        "Islamska republika Afganistan",
        "Albanija",
        "Alžirija",
        "Ljudska demokratična republika Alžirija",
        "Andora",
        "Angola",
        "Republika Angola",
        "Antigva in Barbuda",
        "Argentina",
        "Armenija",
        "Republika Armenija",
        "Avstralija",
        "Avstrija",
        "Azerbajdžan",
        "Azerbajdžanska republika",
        "Bahami",
        "Zveza Bahami",
        "Država Bahrajn",
        "Bangladeš",
        "Ljudska republika Bangladeš",
        "Belgija",
        "Kraljevina Belgija",
        "Belize",
        "Belorusija",
        "Benin",
        "Republika Benin",
        "Bocvana",
        "Republika Bocvana",
        "Republika Bolgarija",
        "Bolivija",
        "Republika Bolivija",
        "Brazilija",
        "Federativna republika Brazilija",
        "Brunej",
        "Burkina Faso",
        "Burundi",
        "Republika Burundi",
        "Butan",
        "Ciper",
        "Republika Ciper",
        "Čad",
        "Republika Čad",
        "Češka",
        "Čile",
        "Republika Čile",
        "Črna gora",
        "Republika Črna gora",
        "Kraljevina Danska",
        "Dominika",
        "Zveza Dominika",
        "Džibuti",
        "Republika Džibuti",
        "Egipt",
        "Arabska republika Egipt",
        "Republika Ekvador",
        "Ekvatorialna Gvineja",
        "Eritreja",
        "Estonija",
        "Republika Estonija",
        "Etiopija",
        "Fidži",
        "Filipini",
        "Republika Filipini",
        "Finska",
        "Republika Finska",
        "Francoska republika",
        "Gabon",
        "Gabonska republika",
        "Gambija",
        "Gana",
        "Republika Gana",
        "Grčija",
        "Helenska republika",
        "Grenada",
        "Gvajana",
        "Republika Gvajana",
        "Gvatemala",
        "Republika Gvatemala",
        "Republika Gvineja",
        "Gvineja Bissau",
        "Republika Gvineja Bissau",
        "Republika Haiti",
        "Honduras",
        "Republika Honduras",
        "Hrvaška",
        "Indija",
        "Republika Indija",
        "Indonezija",
        "Republika Indonezija",
        "Republika Irak",
        "Iran",
        "Islamska republika Iran",
        "Irska",
        "Republika Islandija",
        "Italija",
        "Italijanska republika",
        "Izrael",
        "Jamajka",
        "Japonska",
        "Jemen",
        "Republika Jemen",
        "Jordanija",
        "Južna Afrika",
        "Republika Južna Afrika",
        "Južna Koreja",
        "Kambodža",
        "Kraljevina Kambodža",
        "Kamerun",
        "Republika Kamerun",
        "Katar",
        "Država Katar",
        "Kazahstan",
        "Republika Kazahstan",
        "Kenija",
        "Kirgizistan",
        "Kirgiška republika",
        "Kiribati",
        "Kitajska",
        "Kolumbija",
        "Republika Kolumbija",
        "Komori",
        "Kongo",
        "Republika Kongo",
        "Demokratična republika Kongo",
        "Republika Kostarika",
        "Kuba",
        "Republika Kuba",
        "Kuvajt",
        "Laos",
        "Laoška ljudska demokratična republika",
        "Latvija",
        "Lesoto",
        "Kraljevina Lesoto",
        "Libanon",
        "Libanonska republika",
        "Republika Liberija",
        "Libija",
        "Libijska arabska džamahirija",
        "Lihtenštajn",
        "Kneževina Lihtenštajn",
        "Litva",
        "Republika Litva",
        "Veliko vojvodstvo Luksemburg",
        "Madagaskar",
        "Republika Madagaskar",
        "Republika Madžarska",
        "Makedonija",
        "Republika Makedonija",
        "Malavi",
        "Maldivi",
        "Republika Maldivi",
        "Malezija",
        "Mali",
        "Republika Mali",
        "Republika Malta",
        "Maroko",
        "Kraljevina Maroko",
        "Marshallovi otoki",
        "Mauritius",
        "Republika Mauritius",
        "Mavretanija",
        "Mehika",
        "Združene mehiške države",
        "Mikronezija",
        "Mjanmar",
        "Zveza Mjanmar",
        "Moldavija",
        "Moldavija, Republika",
        "Kneževina Monako",
        "Mongolija",
        "Mozambik",
        "Republika Mozambik",
        "Republika Namibija",
        "Nauru",
        "Republika Nauru",
        "Nemčija",
        "Nepal",
        "Kraljevina Nepal",
        "Niger",
        "Republika Niger",
        "Nigerija",
        "Nikaragva",
        "Republika Nikaragva",
        "Nizozemska",
        "Norveška",
        "Kraljevina Norveška",
        "Nova Zelandija",
        "Oman",
        "Pakistan",
        "Islamska republika Pakistan",
        "Palau",
        "Republika Palau",
        "Republika Panama",
        "Papua Nova Gvineja",
        "Paragvaj",
        "Peru",
        "Republika Peru",
        "Poljska",
        "Republika Poljska",
        "Portugalska republika",
        "Romunija",
        "Ruanda",
        "Republika Ruanda",
        "Ruska federacija",
        "Saint Kitts in Nevis",
        "Saint Lucia",
        "Salomonovi otoki",
        "Salvador",
        "Republika Salvador",
        "San Marino",
        "Sao Tome in Principe",
        "Demokratična republika Sao Tome in Principe",
        "Kraljevina Saudova Arabija",
        "Sejšeli",
        "Republika Sejšeli",
        "Republika Senegal",
        "Severna Koreja",
        "Sierra Leone",
        "Republika Sierra Leone",
        "Singapur",
        "Sirija",
        "Sirska arabska republika",
        "Slonokoščena obala",
        "Slovaška",
        "Slovaška republika",
        "Slovenija",
        "Republika Slovenija",
        "Somalska demokratična republika",
        "Srbija",
        "Republika Srbija",
        "Sudan",
        "Republika Sudan",
        "Surinam",
        "Republika Surinam",
        "Svazi",
        "Španija",
        "Kraljevina Španija",
        "Šrilanka",
        "Švedska",
        "Kraljevina Švedska",
        "Švica",
        "Tadžikistan",
        "Republika Tadžikistan",
        "Tajska",
        "Tajvan",
        "Tajvan, Provinca Kitajske",
        "Tanzanija",
        "Togo",
        "Togoška republika",
        "Tonga",
        "Kraljevina Tonga",
        "Republika Trinidad in Tobago",
        "Tunizija",
        "Republika Tunizija",
        "Republika Turčija",
        "Turkmenistan",
        "Tuvalu",
        "Uganda",
        "Ukrajina",
        "Urugvaj",
        "Vzhodna republika Urugvaj",
        "Uzbekistan",
        "Vanuatu",
        "Republika Vanuatu",
        "Vatikan",
        "Velika Britanija",
        "Združeno kraljestvo",
        "Venezuela",
        "Republika Venezuela",
        "Vietnam",
        "Vzhodni Timor",
        "Demokratična republika Vzhodni Timor",
        "Samoa",
        "Neodvisna država Zahodna Samoa",
        "Zambija",
        "Združene države Amerike",
        "Združene države",
        "Združeni arabski emirati",
        "Zelenortski otoki",
    )

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def street_name(self) -> str:
        return self.random_element(self.streets)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ar_JO`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Jordan
    """

    license_formats = (
        "{{initials}}-####",
        "{{initials}}-#####",
    )

    def initials(self) -> str:
        """Generate an initial number for license plates."""
        return self.random_element(
            [
                "1",  # Ministers
                "2",
                "3",  # Parliament
                "5",  # General Government
                "6",  # Aqaba free zone
                "7",
                "8",  # Diplomatic
                "9",  # Temporary
                "10",
                "23",  # Passenger cars
                "38",
                "39",  # Crew cabs
                "41",
                "42",  # Light goods vehicles
                "44",  # Tractors
                "46",  # Motorcycles and scooters
                "50",  # Taxi
                "56",  # Small buses
                "58",  # Coaches
                "60",  # HGVs
                "70",  # Rental Cars
                "71",  # Trailer
                "90",  # Army
                "95",  # Ambulance
                "96",  # Gendarmerie
                "99",  # Police
            ]
        )

    def license_plate(self) -> str:
        """Generate a license plate."""
        pattern: str = self.random_element(self.license_formats)
        return self.numerify(self.generator.parse(pattern))



# coding=utf-8
import ast
import traceback

from collections import OrderedDict


class SampleCodeValidator(ast.NodeVisitor):
    """
    Class that checks if a string is a valid and "safe" Python expression

    What is considered "safe" for this class is limited to the context of generating
    provider method sample code and output for documentation purposes. The end goal
    is to pass a command string to `eval()` should the string pass the validation
    performed by this class.

    The main assumption this class will make is that the command string passed during
    class instantiation will always be in the form "{generator}.{method}({arguments})".
    In said form, {generator} is a `Generator` object variable that already exists
    within the scope where `eval()` will be called, {method} will be the provider
    method name which is also available within the `eval()` scope, and {arguments}
    will be sample arguments parsed from docstrings. This means that {arguments} can
    potentially be used as a vector for code injection.

    In order to neuter the impact of code injection, the following validation steps
    will be applied:

    - The command string is parsed using 'eval' mode, meaning expressions only.
    - The command string can only have whitelisted code elements. See `_whitelisted_nodes`.
    - The command string can only have one instance of variable access.
    - The command string can only have one instance of attribute access.
    - The command string can only have one instance of a function/method call.
    - The argument values in the command string can only be literals.
    - The only literals allowed are numbers (integers, floats, or complex numbers),
      strings (but not f-strings), bytes, lists, tuples, sets, dictionaries, True,
      False, and None.

    There is, however, an exception. In order to accommodate sample code with custom
    probability distribution, variable access to `OrderedDict` will not count against
    the maximum limit of variable access, and invoking `OrderedDict` constructor calls
    will not count against the maximum limit of function/method calls. In order to
    neuter the impact of code injection, please ensure that `OrderedDict` refers to
    the standard library's `collections.OrderedDict` within the `eval()` scope before
    passing the command string to `eval()` for execution. This can be done in code review.
    """

    _whitelisted_nodes = (
        # Code elements related to function calls and variable and attribute access
        ast.Expression,
        ast.Call,
        ast.Attribute,
        ast.Name,
        ast.Load,
        ast.keyword,
        # Code elements representing whitelisted literals
        ast.Num,
        ast.Str,
        ast.Bytes,
        ast.List,
        ast.Tuple,
        ast.Set,
        ast.Dict,
        ast.NameConstant,
    )

    _max_function_call_count = 1
    _max_attribute_access_count = 1
    _max_variable_access_count = 1

    def __init__(self, command):
        self._errors = set()
        self._function_call_count = 0
        self._attribute_access_count = 0
        self._variable_access_count = 0
        self._command = command

        try:
            self._tree = ast.parse(command, mode="eval")
        except (SyntaxError, ValueError):
            self._log_error(traceback.format_exc())
        else:
            self._validate()

    @property
    def errors(self):
        return self._errors

    def _is_whitelisted(self, node):
        return isinstance(node, self._whitelisted_nodes)

    def _log_error(self, msg):
        self._errors.add(msg)

    def _validate(self):
        self.visit(self._tree)

    def _is_node_using_ordereddict(self, node):
        is_valid = False

        # If instance of function call, check if it is a call to the OrderedDict constructor
        if isinstance(node, ast.Call):
            is_valid = self._is_node_using_ordereddict(node.func)

        # If instance of variable access, check if it is
        elif isinstance(node, ast.Name) and node.id == OrderedDict.__name__:
            is_valid = True

        return is_valid

    def visit(self, node):
        # Check if code element type is allowed
        if not self._is_whitelisted(node):
            msg = "Code element `%s` is not allowed." % node.__class__.__name__
            self._log_error(msg)

        return super().visit(node)

    def visit_Call(self, node):
        if not self._is_node_using_ordereddict(node):
            # There can only be one instance of a function call
            if self._function_call_count < self._max_function_call_count:
                self._function_call_count += 1
            else:
                msg = "There can only be one instance of a function/method call."
                self._log_error(msg)

        # Proceed to child nodes
        self.generic_visit(node)

    def visit_Attribute(self, node):
        # There can only be one instance of attribute access
        if self._attribute_access_count < self._max_attribute_access_count:
            self._attribute_access_count += 1
        else:
            msg = "There can only be one instance of attribute access."
            self._log_error(msg)

        # Proceed to child nodes
        self.generic_visit(node)

    def visit_Name(self, node):
        if not self._is_node_using_ordereddict(node):
            # There can only be one instance of variable access
            if self._variable_access_count < self._max_variable_access_count:
                self._variable_access_count += 1
            else:
                msg = "There can only be one instance of variable access."
                self._log_error(msg)

        # Proceed to child nodes
        self.generic_visit(node)



from .. import Provider as CurrencyProvider

# Names taken from https://www.iban.se/currency-codes


class Provider(CurrencyProvider):
    # Format: (code, name)
    currencies = (
        ("AED", "UAE Dirham"),
        ("AFN", "Afghani"),
        ("ALL", "Lek"),
        ("AMD", "Armenisk Dram"),
        ("ANG", "Gulden från Nederländska Antillerna"),
        ("AOA", "Kwanza"),
        ("ARS", "Argentinsk Peso"),
        ("AUD", "Australisk Dollar"),
        ("AWG", "Arubisk Florin"),
        ("AZN", "Azerbajdzjansk Manat"),
        ("BAM", "Konvertibel Mark"),
        ("BBD", "Barbadosdollar"),
        ("BDT", "Taka"),
        ("BGN", "Bulgarisk Lev"),
        ("BHD", "Bahraini Dinar"),
        ("BIF", "Burundi-franc"),
        ("BMD", "Bermuda-dollar"),
        ("BND", "Brunei-dollar"),
        ("BOB", "Boliviano"),
        ("BOV", "Mvdol"),
        ("BRL", "Brasilisk Real"),
        ("BSD", "Bahamasdollar"),
        ("BTN", "Ngultrum"),
        ("BWP", "Pula"),
        ("BYR", "Vitrysk Rubel"),
        ("BZD", "Belize-dollar"),
        ("CAD", "Kanadensisk Dollar"),
        ("CDF", "Kongolesisk Franc"),
        ("CHE", "WIR Euro"),
        ("CHF", "Schweizerfranc"),
        ("CHW", "WIR Franc"),
        ("CLF", "Unidad de Fomento"),
        ("CLP", "Chilensk Peso"),
        ("CNY", "Yuan Renminbi"),
        ("COP", "Colombiansk Peso"),
        ("COU", "Unidad de Valor Real"),
        ("CRC", "Costa Ricansk Colon"),
        ("CUC", "Peso Convertible"),
        ("CUP", "Kubansk Peso"),
        ("CVE", "Kap Verde Escudo"),
        ("CZK", "Tjeckisk Koruna"),
        ("DJF", "Djibouti-franc"),
        ("DKK", "Dansk Krone"),
        ("DOP", "Dominicansk Peso"),
        ("DZD", "Algerisk Dinar"),
        ("EGP", "Egyptiskt pund"),
        ("ERN", "Nakfa"),
        ("ETB", "Etiopisk Birr"),
        ("EUR", "Euro"),
        ("FJD", "Fiji Dollar"),
        ("FKP", "Pund från Falklandöarna"),
        ("GBP", "Pund Sterling"),
        ("GEL", "Lari"),
        ("GHS", "Ghana Cedi"),
        ("GIP", "Gibraltar-pund"),
        ("GMD", "Dalasi"),
        ("GNF", "Guinea-franc"),
        ("GTQ", "Quetzal"),
        ("GYD", "Guyana-dollar"),
        ("HKD", "Hong Kong-dollar"),
        ("HNL", "Lempira"),
        ("HRK", "Kuna"),
        ("HTG", "Gourde"),
        ("HUF", "Forint"),
        ("IDR", "Rupiah"),
        ("ILS", "Ny Israelisk Shekel"),
        ("INR", "Indisk Rupie"),
        ("IQD", "Irakisk Dinar"),
        ("IRR", "Iransk Rial"),
        ("ISK", "Isländsk Krona"),
        ("JMD", "Jamaica-dollar"),
        ("JOD", "Jordanisk Dinar"),
        ("JPY", "Yen"),
        ("KES", "Kenyansk Shilling"),
        ("KGS", "Som"),
        ("KHR", "Riel"),
        ("KMF", "Comoros-franc"),
        ("KPW", "Nordkoreansk Won"),
        ("KRW", "Won"),
        ("KWD", "Kuwaiti Dinar"),
        ("KYD", "Caymanöar-dollar"),
        ("KZT", "Tenge"),
        ("LAK", "Kip"),
        ("LBP", "Libanesiskt pund"),
        ("LKR", "Sri Lanka Rupie"),
        ("LRD", "Liberiansk Dollar"),
        ("LSL", "Loti"),
        ("LYD", "Libysk Dinar"),
        ("MAD", "Marockansk Dirham"),
        ("MDL", "Moldavisk Leu"),
        ("MGA", "Malagasy Ariary"),
        ("MKD", "Denar"),
        ("MMK", "Kyat"),
        ("MNT", "Tugrik"),
        ("MOP", "Pataca"),
        ("MRO", "Ouguiya"),
        ("MUR", "Mauritius Rupie"),
        ("MVR", "Rufiyaa"),
        ("MWK", "Kwacha"),
        ("MXN", "Mexikansk Peso"),
        ("MXV", "Mexikansk Unidad de Inversion (UDI)"),
        ("MYR", "Malaysisk Ringgit"),
        ("MZN", "Mozambique Metical"),
        ("NAD", "Namibia Dollar"),
        ("NGN", "Naira"),
        ("NIO", "Cordoba Oro"),
        ("NOK", "Norsk Krone"),
        ("NOK", "Norwegian Krone"),
        ("NPR", "Nepalesisk Rupie"),
        ("NZD", "Nya Zealand-dollar"),
        ("OMR", "Rial Omani"),
        ("PAB", "Balboa"),
        ("PEN", "Nuevo Sol"),
        ("PGK", "Kina"),
        ("PHP", "Filippinsk Peso"),
        ("PKR", "Pakistansk Rupie"),
        ("PLN", "Zloty"),
        ("PYG", "Guarani"),
        ("QAR", "Qatari Rial"),
        ("RON", "Rumänsk Leu"),
        ("RSD", "Serbisk Dinar"),
        ("RUB", "Rysk Rubel"),
        ("RWF", "Rwanda Franc"),
        ("SAR", "Saudi Riyal"),
        ("SBD", "Dollar från Salomonöarna"),
        ("SCR", "Seychell-rupie"),
        ("SDG", "Sudanesiskt pund"),
        ("SEK", "Svensk Krona"),
        ("SGD", "Singapore Dollar"),
        ("SHP", "Saint Helena pund"),
        ("SLL", "Leone"),
        ("SOS", "Somalisk Shilling"),
        ("SRD", "Surinam Dollar"),
        ("SSP", "Sydsudanesiskt pund"),
        ("STD", "Dobra"),
        ("SVC", "El Salvador Colon"),
        ("SYP", "Syriskt pund"),
        ("SZL", "Lilangeni"),
        ("THB", "Baht"),
        ("TJS", "Somoni"),
        ("TMT", "Turkmenistansk Ny Manat"),
        ("TND", "Tunisisk Dinar"),
        ("TOP", "Pa’anga"),
        ("TRY", "Turkisk Lira"),
        ("TTD", "Trinidad och Tobago Dollar"),
        ("TWD", "Ny Taiwanesisk Dollar"),
        ("TZS", "Tanzanisk Shilling"),
        ("UAH", "Hryvnia"),
        ("UGX", "Uganda Shilling"),
        ("USD", "US Dollar"),
        ("USN", "US Dollar (Nästa dag)"),
        ("UYI", "Uruguay Peso en Unidades Indexadas (URUIURUI)"),
        ("UYU", "Peso Uruguayo"),
        ("UZS", "Uzbekistansk Sum"),
        ("VEF", "Bolivar"),
        ("VND", "Dong"),
        ("VUV", "Vatu"),
        ("WST", "Tala"),
        ("XAF", "CFA Franc BEAC"),
        ("XCD", "East Caribbean Dollar"),
        ("XDR", "SDR (Särskild dragningsrätt)"),
        ("XOF", "CFA Franc BCEAO"),
        ("XPF", "CFP Franc"),
        ("XSU", "Sucre"),
        ("XUA", "ADB Beräkningsenhet"),
        ("YER", "Yemeni Rial"),
        ("ZAR", "Rand"),
        ("ZMW", "Zambian Kwacha"),
        ("ZWL", "Zimbabwe Dollar"),
    )
